/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Kyle Nitzsche <kyle.nitzsche@canonical.com>
 *              Scott Sweeny <scott.sweeny@canonical.com>*
 */

#ifndef AGGSCOPEQUERY_H
#define AGGSCOPEQUERY_H

#include "scope.h"
#include "aggchildscope.h"
#include "resultforwarder.h"

#include <functional>

#include <unity/scopes/SearchQueryBase.h>
#include <unity/scopes/ReplyProxyFwd.h>
#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/ChildScope.h>
#include <unity-scopes.h>

class Query : public unity::scopes::SearchQueryBase
{
public:
    Query(const unity::scopes::CannedQuery &query,
                    const unity::scopes::SearchMetadata &metadata,
                    const std::string scope_dir,
                    const std::string cache_dir,
                    unity::scopes::RegistryProxy const& registry,
                    AggScope * agg_scope
                   );
    ~Query();
    virtual void cancelled() override;

    virtual void run(unity::scopes::SearchReplyProxy const& upstream_reply) override;

    bool using_departments = true;

private:
    std::string scope_id;//this scope's for use in log messages
    struct category {
        std::string id;
        std::string title;
        std::string dept_id;
        int cardinality = -1;
        bool use_incoming_title = false;
        std::string category_link_to_child_specified;
        std::string renderer;
        std::string search_renderer;
        std::vector<std::pair<std::string,std::string>> scopes; // id and local_id of scopes
        std::vector<std::string> keywords; // keywords in this cat
        std::vector<std::pair<std::string,std::string>> keyword_ids; // id and local id of keywords
    };
    std::map<std::string,std::string> localId_defaultQryStr;
    std::map<std::string,std::string> scopeLocalId_childDept_m; // local_id of scope to scope's child department
    std::vector<std::shared_ptr<category>> categories;
    std::map<std::string,std::shared_ptr<category>> id_category_m;
    std::map<std::string,std::string> categoryId_titleMsgid;
    std::vector<std::string> categoryIds;
    std::map<std::string,std::string> categoryId_linkToChildSpecified;
    std::map<std::string,std::string> categoryId_first_result_renderers; //shard kw to (optional) declared first restult category renderers
    std::map<std::string,std::string> categoryId_firstRdrs_catId2commonId;
    std::map<std::string,std::string> categoryId_renderer; //shard kw to (optional) declared category renderer
    std::map<std::string,std::string> categoryId_surfaceRdrs_catId2commonId;
    std::map<std::string,std::string> categoryId_search_renderer; //shard kw to (optional) declared category renderer when searching
    std::map<std::string,bool> categoryId_isFirstResult;
    std::map<std::string,bool> categoryId_isSecondResult;
    std::list<std::string> declared_category_keywords;// the declared list of category keywords, in declared order

    int get_setting_cardinality();
    std::vector<int> setting_cardinalities; // values defined in qQuery constructor and settings.ini
    AggScope *agg_scope_;
    std::map<std::string,std::string> common_templates; // common_template id to common_template in json
    std::map<std::string,std::shared_ptr<std::map<std::string,std::vector<std::string>>>> common_templates_fallbacks; // common_template id key to result field key to list of fallback field keys in result
    std::map<std::string,us::Category::SCPtr> registered_common_categories;//id is unique to scope and common category id.
    void check_result_fallbacks(unity::scopes::CategorisedResult &res, std::string cat_id);
    struct child_scope {
        //these are the json keys for each child scope
        std::string id;//required in json
        std::string local_id;//required in json
        bool link_to_child = true;//when false, category has no link to child
        bool source_finder = false; //means this scope is to search the store for sources
        std::string category_title = "";//optional in json required
        bool using_category_title_incoming = false;//when true, set cat title from first incoming result
        bool using_category_title_display_name = false;//when true, set cat title from result scope's display name
        std::string category_title_ = "";//use in json when title i18ned
        std::string first_result_template = "";//required in json
        std::string surface_template = "";//required in json
        std::string search_template = "";//if not present in json, surface_template used for search
        int cardinality = 6;//optional in json, else use this default
        std::string child_category = "";//optional in json
        int child_category_max_results = 6;//optional in json else use this default when child_category used
        std::string child_department = "";//optional in json
        std::string department = "";//optional in json
        bool only_in_search = false;//optional in json
        //the rest are NOT json keys
        bool using_child_category_max_results = false;//NOT A JSON KEY
        bool overriding_surface_template = false;//NOT A JSON KEY
        bool overriding_search_template = false;//NOT A JSON KEY
        std::vector<std::shared_ptr<std::pair<std::string,std::string>>> swap_result_attributes;
        std::pair<std::string,std::string> if_category_use_template;
        std::map<std::string, std::string> result_category_id_to_common_template;//id is result category id. maps to id to use in common_templates
    };
    std::vector<std::shared_ptr<child_scope>> child_scopes;// pulled from child_sccopes.json
    std::map<std::string,std::shared_ptr<child_scope>> child_scopes_m;//id to child_scope
    std::vector<std::string> current_scopes;// scope local_id
    std::vector<std::string> current_scopeIds;// keyword cate scope ids to prevent dups
    void set_scope_order();
    std::vector<std::string> declared_order;// scope local_id or keyword
    std::vector<std::string> scopes_ordered;// scope local id or keyword
    /*
    for type_ids_m:
    declared scope:
        key: local_id
        vector: local_id (vector has size of 1)
    declared keyword:
        key: keyword
        vector: scope ids (vector size varies)
    declared category keyword:
        key: category
        vector: scope ids (vector size varies)
    */
    std::map<std::string,std::vector<std::string>> type_ids_m;// 


    struct dept {
        std::string id;//actual scope id
        std::string local_id;//locally unique id to allow same scope multiple times
        std::string title;
        std::string keyword;// optional. stores msgid to look up title
        bool root = false;
    };
    std::map<std::string,std::string> localId_id_m;//use scope's local_id to get its fully qualified id
    std::string dept_id_of_root;
    std::string dept_id_of_root_keywords_only;

    std::map<std::string,std::shared_ptr<dept>> depts; // dept id to dept ptr
    struct keyword {
        std::string id; // the keyword string
        int cardinality = -1;
        std::string dept_id;
        std::string dept_title_msgid;
        bool link_to_child = false;
        std::vector<std::string> exclude_scopes; // id of exclude scopes
    };
    std::vector<std::shared_ptr<keyword>> keywords;// pulled from child_sccopes.json
    std::map<std::string, std::shared_ptr<keyword>> id_keyword_map;//keyword id (the keyword string) to keyword instance
    std::map<std::string,std::string> keyword_deptTitleMsgid; // keyword to msgid used to look up its title
    std::map<std::string,std::string> keyword_deptId;

    void getDeclaredChildScopes();
    void make_declared_scopes();
    void make_keyword_scopes();
    void make_categories();
    void load_declarations();
    us::ChildScopeList current_child_scopes;

    std::string first_result_owner; // set to first declaared keyword or scope that declared a first result renderer
    std::list<std::string> declared_keywords;// the declared list of keywords, in declared order
    std::list<std::string> shared_keywords;// the declared list of shared category keywords
    std::map<std::string,std::string> shared_keyword_cats; //kw to category
    std::map<std::string,std::string> keyword_catname_first_result_renderers; //shard kw to (optional) declared first restult category renderers
    std::map<std::string,std::string> keyword_catname_renderer; //shard kw to (optional) declared category renderer
    std::map<std::string,std::string> keyword_catname_search_renderer; //shard kw to (optional) declared category renderer when searching
    std::map<std::string,bool> keywordSharedCat_isFirstResult;
    std::map<std::string,bool> keywordSharedCat_isSecondResult;
    std::map<std::string,std::string> shared_keyword_cat_titlemsgid; //category to tittle for l10n lookup
    std::map<std::string,us::Category::SCPtr> catname_catptr;//store pointer to shared category
    std::map<std::string,bool> sharedcat_catregistered;//stores whether the shared cat's ctegory is registered yet
    void create_departments(us::SearchReplyProxy const &reply_);
    void handle_current_child_scopes(bool empty_search, us::SearchReplyProxy const& upstream_reply);
    void handle_child_scope_results(std::string query_string, bool empty_search, us::SearchReplyProxy const& upstream_reply);
    unity::scopes::RegistryProxy registry_;
    std::vector<us::ScopeProxy> scopes; // will hold all enabled and present child scopes
    std::map<std::string,std::shared_ptr<AggChildScope>> scopes_m;//key is local_id
    std::map<int,std::string> scopes_m_int_localId;
    std::string scope_dir_;
    std::string cache_dir_;
    std::string query_store_; //for searching the store for a given a scope

    void noSources(unity::scopes::SearchReplyProxy const& reply);

    bool no_network_msg = false;
    bool no_network_msg_hints = false;
    bool no_location_msg = false;
    bool no_location_msg_hints = false;
    bool uses_hints = false;
    unity::scopes::ScopeProxy hints_scope;
    bool hints_quickstart = false;
    QString hints_is_hidden = "hints_is_hidden";
    QString firstboot = "firstboot";

    void load_hints();
    bool hints_exists();
    void display_hints_quickstart(us::SearchReplyProxy const& upstream_reply_);
    bool show_hints(us::SearchReplyProxy const& upstream_reply_);
    bool is_hints_quickstart_dismissed();
    void dismiss_hints_quickstart();
    std::string HINTS_SCOPE_ID;//fqn of hints scope
    std::string HINTS_THIS_SCOPE;//tells hints which agg scope this is
    std::string HINTS_HIDE;//tells hints to stop displaying

    bool has_one_source=false;//true means scope is installed
    int keyword_cardinality = 5;

    void load_cardinality_settings();
};

#endif
