/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Kyle Nitzsche <kyle.nitzsche@canonical.com>
 */

#include "aggchildscope.h"
#include "utils.h"
#include <unity/scopes/ScopeMetadata.h>

namespace us = unity::scopes;

AggChildScope::AggChildScope(const std::string id) :
    id_(id)
{
}

void AggChildScope::set_local_id(const std::string local_id)
{
    local_id_ = local_id;
}

std::string AggChildScope::local_id()
{
    return local_id_;
}

void AggChildScope::set_category_title(const std::string title_)
{
    cat_title = title_;
}

std::string AggChildScope::category_title()
{
    return cat_title;
}

void AggChildScope::set_category(us::Category::SCPtr category_)
{
    cat = category_;
}

bool AggChildScope::using_category_title_incoming()
{
    return using_category_title_incoming_;
}

void AggChildScope::set_category_title_incoming(bool b)
{
    using_category_title_incoming_ = b;
}

bool AggChildScope::using_category_title_display_name()
{
    return using_category_title_display_name_;
}

void AggChildScope::set_category_title_display_name(bool b)
{
    using_category_title_display_name_ = b;
}


us::Category::SCPtr AggChildScope::category()
{
    return cat;
}

void AggChildScope::set_first_result_template(const std::string first_result_template)
{
    TEMPLATE_FIRST_RESULT = first_result_template;
}

std::string AggChildScope::first_result_template()
{
    return TEMPLATE_FIRST_RESULT;
}

void AggChildScope::set_surface_template(const std::string surface_template)
{
    TEMPLATE_SURFACE = surface_template;
}

std::string AggChildScope::surface_template()
{
    return TEMPLATE_SURFACE;
}

void AggChildScope::set_override_surface_template(bool b)
{
    override_surface_template_ = b;
}

bool AggChildScope::override_surface_template()
{
    return override_surface_template_;
}

std::string AggChildScope::search_template()
{
    return TEMPLATE_SEARCH;
}

void AggChildScope::set_search_template(const std::string search_template)
{
    TEMPLATE_SEARCH = search_template;
}

void AggChildScope::set_override_search_template(bool b)
{
    override_search_template_ = b;
}

bool AggChildScope::override_search_template()
{
    return override_search_template_;
}

void AggChildScope::set_child_department(const std::string id_)
{
    subsearch_dept_id = id_;
}

std::string AggChildScope::child_department()
{
    return subsearch_dept_id;
}

void AggChildScope::set_department(const std::string id_)
{
    dept_id = id_;
}

std::string AggChildScope::department()
{
    return dept_id;
}

void AggChildScope::set_child_category(const std::string category_id)
{
    category_id_ = category_id;
    using_category = true;
}

std::string AggChildScope::child_category()
{
    return category_id_;
}

void AggChildScope::set_child_category_max_results(int max)
{
    child_category_max_results_ = max;
    using_child_category_max_results_ = true;
}

bool AggChildScope::using_child_category_max_results()
{
    return using_child_category_max_results_;
}

int AggChildScope::child_category_max_results()
{
    return child_category_max_results_;
}

void AggChildScope::set_cardinality(int cardinality)
{
    cardinality_ = cardinality;
}

int AggChildScope::cardinality()
{
    return cardinality_;
}

std::string AggChildScope::id()
{
    return id_;
}

void AggChildScope::set_category_link_to_child(bool b)
{
    using_category_link_to_child = b;
}

bool AggChildScope::category_link_to_child()
{
    return using_category_link_to_child;
}

void AggChildScope::set_proxy(us::RegistryProxy registry)
{

    auto scopes = registry->list();
    us::MetadataMap::const_iterator scope;
    scope = scopes.find(id_);
    if (scope != scopes.end()) {
        set_metadata(registry->get_metadata(id_));
        proxy_ = metadata()->proxy();
        exists_ = true;
    }
}

void AggChildScope::set_metadata(us::ScopeMetadata const & md)
{
    scope_metadata_ = std::make_shared<us::ScopeMetadata>(md);
}

std::shared_ptr<us::ScopeMetadata> AggChildScope::metadata()
{
    return scope_metadata_;
}

bool AggChildScope::enabled(const us::VariantMap settings)
{
    if (! settings.at(id()).get_bool())
    {
        return false;
    }
    return true;
}

us::ScopeProxy AggChildScope::proxy()
{
    return proxy_;
}

void AggChildScope::inc_result_idx()
{
    result_idx_++;
}

int AggChildScope::result_idx()
{
    return result_idx_;
}

void AggChildScope::set_buffer(bool b)
{
    buffer_ = b;
}

bool AggChildScope::buffer()
{
    return buffer_;
}

void AggChildScope::set_category_scope(bool b)
{
    category_scope_ = b;
}

bool AggChildScope::category_scope()
{
    return category_scope_;
}

void AggChildScope::set_category_id(std::string id)
{
    category_scope_id_ = id;
}

std::string AggChildScope::category_id()
{
    return category_scope_id_;
}

void AggChildScope::set_keyword_scope(bool b)
{
    keyword_scope_ = b;
}

bool AggChildScope::keyword_scope()
{
    return keyword_scope_;
}

void AggChildScope::set_keyword(std::string kw)
{
    keyword_ = kw;
}

std::string AggChildScope::keyword()
{
    return keyword_;
}

void AggChildScope::set_keyword_scope_shared_cat_name(std::string cat)
{
    keyword_scope_shared_cat_name_ = cat;
    uses_shared_cat_ = true;
}

std::string AggChildScope::keyword_scope_shared_cat_name()
{
    return keyword_scope_shared_cat_name_;
}

bool AggChildScope::keyword_uses_shared_cat()
{
    return uses_shared_cat_;
}

void AggChildScope::set_source_finder(bool b)
{
    source_finder_ = b;
}

bool AggChildScope::only_in_search()
{
    return only_in_search_;
}

void AggChildScope::set_only_in_search(bool b)
{
    only_in_search_ = b;
}

bool AggChildScope::source_finder()
{
    return source_finder_;
}

void AggChildScope::set_first_common_template_id(std::string id)
{
    first_common_template_id_ = id;
}

std::string AggChildScope::first_common_template_id()
{
    return first_common_template_id_;
}

void AggChildScope::set_surface_common_template_id(std::string id)
{
    surface_common_template_id_ = id;
}

std::string AggChildScope::surface_common_template_id()
{
    return surface_common_template_id_;
}
